import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.semanticweb.owlapi.apibinding.OWLManager;
import org.semanticweb.owlapi.model.*;
import org.semanticweb.owlapi.reasoner.NodeSet;
import org.semanticweb.owlapi.reasoner.OWLReasoner;
import org.semanticweb.owlapi.reasoner.OWLReasonerFactory;
import org.semanticweb.owlapi.reasoner.structural.StructuralReasonerFactory;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;

public class RobotApplication {

    public static Position parseOntology(String zutat) {
        try {
            OWLOntologyManager manager = OWLManager.createOWLOntologyManager();
            OWLOntology o = manager.loadOntologyFromOntologyDocument( RobotApplication.class.getResourceAsStream("positions.owl"));
            OWLReasonerFactory reasonerFactory = new StructuralReasonerFactory();
            OWLReasoner reasoner = reasonerFactory.createReasoner(o);
            reasoner.precomputeInferences();
            OWLDataFactory fac = manager.getOWLDataFactory();

            for (OWLClass c : o.getClassesInSignature()) {
                if (c.getIRI().getFragment().equals("Zutaten")){
                    NodeSet<OWLNamedIndividual> instances = reasoner.getInstances(c, false);
                    //System.out.println("1 " + c.getIRI().getFragment());
                    for (OWLNamedIndividual i : instances.getFlattened()) {
                        //System.out.println("2 " + i.getIRI().getFragment());
                        Set<OWLDataPropertyAssertionAxiom> properties=o.getDataPropertyAssertionAxioms(i);
                        Position pos = new Position();
                        for (OWLDataPropertyAssertionAxiom ax: properties) {
                            if(ax.getProperty().toString().contains("hasZ-coordinate"))
                                pos.setZ(Integer.parseInt(ax.getObject().getLiteral()));
                            if(ax.getProperty().toString().contains("hasX-coordinate"))
                                pos.setX(Integer.parseInt(ax.getObject().getLiteral()));
                            if(ax.getProperty().toString().contains("hasY-coordinate"))
                                pos.setY(Integer.parseInt(ax.getObject().getLiteral()));
                          //  System.out.println("xx" + ax.getObject().getLiteral()  + " " + ax.getProperty().toString());
                        }
                        if(i.getIRI().getFragment().toString().equals(zutat))
                            return pos;
                    }
                }
            }
        } catch (OWLOntologyCreationException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static void main(String [] args) {
        String url = "http://austria.omilab.org/omirob/dobot2/rest/";
        String token;

        System.out.println("Use argument 'help' to get help!");

        if(args[0].equals("help")) {
            System.out.println("How to use: Robot.jar <DoBotURL> <Tokens> beilagen <beilage1> <beilage2> burger <burgerzutat1> <burgerzutat2>");
            System.exit(0);
        }

        boolean fbeilagen = false;
        boolean fburger = false;

        List<String> beilagen = new ArrayList<String>();
        List<String> burger = new ArrayList<String>();
        for(int i=0; i<=args.length-1; i++){
            if(args[i].equals("burger")) {
                fbeilagen = false;
                fburger = true;
            }
            if(args[i].equals("beilagen")) {
                fbeilagen = true;
                fburger = false;
            }
            if(fburger)
                burger.add(args[i]);
            if(fbeilagen)
                beilagen.add(args[i]);
        }

        url = args[0];
        token = args[1];

        System.out.println("URL: " + url);
        System.out.println("Tokens: " + token);

        beilagen.remove("burger");
        beilagen.remove("beilagen");
        burger.remove("burger");
        burger.remove("beilagen");

        System.out.println("Beilagen to position: " + beilagen.toString());
        System.out.println("Burger parts to position " + burger.toString());

        //parseOntology("Chili").toString());

        Position zielBurger = new Position(215,-250,100);
        Position zielBeilage = new Position(205,-210,100);
        Position zwischenPunkt = new Position(270,0,120);

        //reset
        reset(token,url);

        //positionArm(330,30,70,token,url);

        //position beilagen
        for(String beilage : beilagen) {
            Position pos = parseOntology(beilage);
            positionArm(pos.getX(),pos.getY(),pos.getZ()+20,token,url);
            waitywaity();
            positionArm(pos.getX(),pos.getY(),pos.getZ(),token,url);
            waitywaity();
            grab(true,token,url);
            waitywaity();
            positionArm(pos.getX(),pos.getY(),pos.getZ()+20,token,url);
            waitywaity();
            positionArm(zwischenPunkt.getX(),zwischenPunkt.getY(),zwischenPunkt.getZ(),token,url);
            waitywaity();
            positionArm(zielBeilage.getX(),zielBeilage.getY(),zielBeilage.getZ(),token,url);
            waitywaity();
            grab(false,token,url);
            waitywaity();
            positionArm(zwischenPunkt.getX(),zwischenPunkt.getY(),zwischenPunkt.getZ(),token,url);
        }

        //position burger
        for(String burg : burger) {
            Position pos = parseOntology(burg);
            positionArm(pos.getX(),pos.getY(),pos.getZ()+20,token,url);
            waitywaity();
            positionArm(pos.getX(),pos.getY(),pos.getZ(),token,url);
            waitywaity();
            grab(true,token,url);
            waitywaity();
            positionArm(pos.getX(),pos.getY(),pos.getZ()+20,token,url);
            waitywaity();
            positionArm(zwischenPunkt.getX(),zwischenPunkt.getY(),zwischenPunkt.getZ(),token,url);
            waitywaity();
            positionArm(zielBurger.getX(),zielBurger.getY(),zielBurger.getZ(),token,url);
            waitywaity();
            grab(false,token,url);
            waitywaity();
            positionArm(zwischenPunkt.getX(),zwischenPunkt.getY(),zwischenPunkt.getZ(),token,url);
        }

    }

    public static void waitywaity() {
        try {
            Thread.sleep(1000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }


    public static void positionArm(int x, int y, int z, String token, String url) {

        String posUrl = url + "positionXYZ";

        HttpClient client = HttpClientBuilder.create().build();
        HttpPut put = new HttpPut(posUrl);

        put.setHeader("User-Agent", "CMKE project");
        put.setHeader("Content-Type","application/json; charset=utf8");
        put.setHeader("Authorization","Bearer " + token);

        String result = null;
        try {
            String posData = "{\n" +
                    "   \"x\": "+x+",\n" +
                    "   \"y\": "+y+",\n" +
                    "   \"z\": "+z+"\n" +
                    "}";
            HttpEntity entity = new ByteArrayEntity(posData.getBytes("UTF-8"));
            put.setEntity(entity);
            HttpResponse response = client.execute(put);
            result = EntityUtils.toString(response.getEntity());
            System.out.println(new Date().toString() + " Position x:" + x + " y: " + y + " z: " + z + " --> " + response.getStatusLine());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void reset(String token, String url) {

        String resetUrl = url + "reset";

        HttpClient client = HttpClientBuilder.create().build();
        HttpPost post = new HttpPost(resetUrl);

        post.setHeader("User-Agent", "CMKE project");
        post.setHeader("Content-Type","application/json; charset=utf8");
        post.setHeader("Authorization","Bearer " + token);

        String result = null;
        try {
            HttpResponse response = client.execute(post);
            result = EntityUtils.toString(response.getEntity());
            System.out.println(new Date().toString() + " RESET --> " + response.getStatusLine());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void grab(Boolean what, String token, String url) {

        String grabUrl = url + "grabOn";

        HttpClient client = HttpClientBuilder.create().build();
        HttpPost post = new HttpPost(grabUrl);

        // add header
        post.setHeader("User-Agent", "CMKE project");
        post.setHeader("Content-Type","application/json; charset=utf8");
        post.setHeader("Authorization","Bearer " + token);

        String result = null;
        try {
            String grabData = what.toString();
            HttpEntity entity = new ByteArrayEntity(grabData.getBytes("UTF-8"));
            post.setEntity(entity);
            HttpResponse response = client.execute(post);
            result = EntityUtils.toString(response.getEntity());
            System.out.println(new Date().toString() + " Grab val:" + what + " --> " + response.getStatusLine());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }



}
